import { Injectable } from "@angular/core";
import {
  HttpClient,
  HttpHeaders,
  HttpParams,
  HttpUrlEncodingCodec
} from "@angular/common/http";
import { Observable } from "rxjs";
import { environment } from "../../../environments/environment";

export class CustomHttpUrlEncodingCodec extends HttpUrlEncodingCodec {
  encodeValue(v: string): string {
    return customEncoding(v);
  }
}

function customEncoding(v: string): string {
  return encodeURIComponent(v)
    .replace(/%40/gi, "@")
    .replace(/%24/gi, "$")
    .replace(/%2C/gi, ",")
    .replace(/%3B/gi, ";")
    .replace(/%3D/gi, "=")
    .replace(/%3F/gi, "?")
    .replace(/%2F/gi, "/");
}

@Injectable({
  providedIn: "root"
})
export class ApiBaseService {
  baseUrl = environment.apiBaseURL;
  apiPrefix = "";
  apiVersion = "";

  constructor(private httpClient: HttpClient) { }

  public getFullApiRoute(route: string, id?: number): string {
    let apiRoute;
    if (id) {
      apiRoute = `${this.baseUrl}/${this.apiPrefix}/${this.apiVersion}/${route}/${id}/`;
      // apiRoute = `${route}/${id}`;
    } else {
      // apiRoute = `${this.baseUrl}/${this.apiPrefix}/${this.apiVersion}/${route}/`;
      apiRoute = `${this.baseUrl}/${route}/`;
      // apiRoute = `${route}`;
    }
    return apiRoute;
  }

  private getDefaultHeaders() {
    let headers = new HttpHeaders();
    headers = headers
      .append("Content-Type", "application/json")
      .append("Access-Control-Allow-Methods", "*")
      .append("Access-Control-Allow-Methods", "GET,POST,OPTIONS,DELETE,PUT")
      .append("Access-Control-Allow-Origin", "*")
    // .append(
    //   "Access-Control-Allow-Headers",
    //   "Access-Control-Allow-Headers, Access-Control-Allow-Origin, Access-Control-Request-Method"
    // );
    return headers;
  }

  private getHttpParams(params?) {
    return new HttpParams({
      fromObject: params,
      encoder: new CustomHttpUrlEncodingCodec()
    });
  }

  private getOptions(optionalParams?) {
    const headers = this.getDefaultHeaders();
    const httpParams = this.getHttpParams(optionalParams);
    return { params: httpParams, headers: headers };
  }

  /**
   * GET
   */
  public get<T>(route: string, params?): Observable<T> {
    return this.httpClient.get<T>(
      this.getFullApiRoute(route),
      this.getOptions(params)
    );
  }

  public getById<T>(route: string, id: number, params?): Observable<T> {
    return this.httpClient.get<T>(
      this.getFullApiRoute(route, id),
      this.getOptions(params)
    );
  }

  /**
   * POST
   */
  public post<T>(route: string, data?, params?): Observable<T> {
    return this.httpClient.post<T>(
      this.getFullApiRoute(route),
      JSON.stringify(data),
      this.getOptions(params)
    );
  }

  public postById<T>(route: string, id: number, data?, params?): Observable<T> {
    return this.httpClient.post<T>(
      this.getFullApiRoute(route, id),
      data,
      this.getOptions(params)
    );
  }


  /**
   * PUT
   */
  public put(route: string, data, params?) {
    return this.httpClient.put(
      this.getFullApiRoute(route),
      JSON.stringify(data),
      this.getOptions(params)
    );
  }

  public putById(route: string, id: number, data?, params?) {
    return this.httpClient.put(
      this.getFullApiRoute(route, id),
      data,
      this.getOptions(params)
    );
  }

  /**
   * DELETE
   */
  public delete(route: string, params?) {
    return this.httpClient.delete(
      this.getFullApiRoute(route),
      this.getOptions(params)
    );
  }

  public deleteById(route: string, id: number, params?) {
    return this.httpClient.delete(
      this.getFullApiRoute(route, id),
      this.getOptions(params)
    );
  }
}
